#!/usr/bin/env python
# -*- coding: utf-8 -*-
import schedule
import constants
import time
import daemon.pidfile
import daemon
import os
import argparse
from thunderbolt100k.libs.common import write_conf, load_conf
import sys
from thunderbolt100k import __VERSION__
from thunderbolt100k.widgets import *  # Import all the widgets


def widget_main(widget):
    result = widget.fetch()
    if result:
        with file(constants.DATA_PATH_FORMAT.format(widget.__name__.replace('thunderbolt100k.widgets.', '')), mode='w+') as f:
            f.write(result)


def polling():
    for m in sys.modules['thunderbolt100k.widgets'].modules:
        if m.endswith('__init__.py'):
            continue

        widget_name = os.path.basename(m).replace('.py', '')
        widget = getattr(sys.modules['thunderbolt100k.widgets'], widget_name)
        if constants.CONFIG.get('{0}_INTERVAL'.format(widget.__name__.upper())):
            interval = int(constants.CONFIG.get('{0}_INTERVAL'.format(widget_name.upper())))
        else:
            interval = 5  # Default is 5 min if not specified
        schedule.every(interval).minutes.do(widget_main, widget)
        widget_main(widget)  # Run the function instantly and then schedule
    while True:
        schedule.run_pending()
        time.sleep(60)


def init_zshrc():
    import fileinput

    exist_conf = load_conf()
    if exist_conf:
        print('Thunderbolt100k has initialized before, delete the corresponding contents in `~/.zshrc` and try again')
        return

    # Set default settings
    print('Initializing default settings...')
    write_conf('VERSION', __VERSION__, first_write=True)
    for key in constants.DEFAULT_CONFIG:
        write_conf(key, constants.DEFAULT_CONFIG[key])

    # Bind polling command to a custom PL9K element
    write_conf('POWERLEVEL9K_CUSTOM_POLLING', '"thunderbolt100k polling"', without_prefix=True)

    elements_names = ['custom_polling']
    # Set PL9K custom command
    for m in sys.modules['thunderbolt100k.widgets'].modules:
        if m.endswith('__init__.py'):
            continue
        widget_name = os.path.basename(m).replace('.py', '')
        print('Initializing [{0}] widget...'.format(widget_name))
        widget = getattr(sys.modules['thunderbolt100k.widgets'], widget_name)
        write_conf('POWERLEVEL9K_CUSTOM_{0}'.format(widget_name.upper()),
                   '"thunderbolt100k display {0}"'.format(widget_name), without_prefix=True)

        elements_names.append('custom_{0}'.format(widget_name))
        # Ask for extra info for each widgets
        result = widget.user_input()
        for k in result:
            write_conf(k, result[k])

    # Add the custom elements to PL9K
    home = os.path.expanduser("~")
    path = os.path.join(home, '.zshrc')
    for line in fileinput.input(path, inplace=True):
        if line.startswith('POWERLEVEL9K_RIGHT_PROMPT_ELEMENTS'):
            print(line.replace(')', ' {0})'.format(' '.join(elements_names))).rstrip())
        else:
            print(line.rstrip())
    fileinput.close()

    print('Initialization done! Open a new shell session and enjoy the view!')
    print('You may also want to rearrange the widgets locations by editing `POWERLEVEL9K_LEFT_PROMPT_ELEMENTS` and \n' +
          '`POWERLEVEL9K_RIGHT_PROMPT_ELEMENTS` in `~/.zshrc` file')
    print('If you want to set configurations for THUNDERBOLT100K, please refer to https://github.com/cuyu/thunderbolt100k#configuration')


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description='A cmdline tool to enhance PL9K theme for ZSH\n')
    subparsers = parser.add_subparsers(help='Use {subcommand} -h for each subcommand\'s optional arguments details',
                                       dest='command')
    subparsers.add_parser('init', help='Init the settings in `~/.zshrc` file')
    subparsers.add_parser('polling', help='Start the polling daemon process')
    display_parser = subparsers.add_parser('display', help='Print the corresponding info on the terminal')
    display_parser.add_argument('widget', help='The widget to display, e.g. weather')

    args = parser.parse_args()
    if args.command == 'polling':
        with daemon.DaemonContext(pidfile=daemon.pidfile.PIDLockFile(constants.PID_PATH)):
            polling()
    elif args.command == 'display':
        widgets = sys.modules['thunderbolt100k.widgets']
        assert hasattr(widgets, args.widget), 'There is no widget called {0}'.format(args.widget)
        assert hasattr(getattr(widgets, args.widget),
                       'display'), 'The widget {0} must contains a `display` method'.format(args.widget)
        result = getattr(widgets, args.widget).display()
        if result:
            print(result)
    elif args.command == 'init':
        init_zshrc()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os


def pl9k_color(color):
    return '%F{' + color + '}'


# Assume the conf location is '~/.zshrc'
def load_conf():
    home = os.path.expanduser("~")
    path = os.path.join(home, '.zshrc')
    config = dict()
    with file(path, mode='r') as f:
        while True:
            line = f.readline()
            if not line:
                break
            if line.startswith('THUNDERBOLT100K'):
                key, value = line.split('=')
                # Remove the prefix in key and the quotation marks in the value
                config[key.split('THUNDERBOLT100K_')[1]] = value.replace('"', '').replace("'", '').strip()
    return config


def write_conf(key, value, replace_exist=False, first_write=False, without_prefix=False):
    home = os.path.expanduser("~")
    path = os.path.join(home, '.zshrc')

    if without_prefix:
        template = '{0}={1}\n'
    else:
        template = 'THUNDERBOLT100K_{0}={1}\n'

    if first_write:
        with file(path, mode='a') as f:
            f.write("############################\n# For THUNDERBOLT100K\n")
        write_conf(key, value, replace_exist)
    else:
        if replace_exist:
            raise NotImplementedError
        else:
            with file(path, mode='a') as f:
                f.write(template.format(key, value))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime


# Compare the given time stamp with current time and return how long has been passed since then
def compare_time(time_stamp, time_format):
    now = datetime.datetime.now()
    target_time = datetime.datetime.strptime(time_stamp, time_format)

    delta_time = now - target_time

    delta_minute = (delta_time.seconds / 60) % 60
    delta_hour = (delta_time.seconds / 3600) % 24
    delta_day = delta_time.days

    return delta_minute, delta_hour, delta_day


if __name__ == '__main__':
    # Example: python compare_time.py '2017-07-07 12:00' '%Y-%m-%d %H:%M'
    import sys

    print compare_time(sys.argv[1].replace('"', '').replace("'", ''), sys.argv[2])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from libs.common import load_conf

CONFIG = dict()
DEFAULT_CONFIG = {
    'WEATHER_INTERVAL': '5',
    'WEATHER_INFO': '(condition temp last_update)',
    'WEATHER_RAIN_COLOR': '153',
    'WEATHER_SUN_COLOR': '172',
    'WEATHER_CLOUD_COLOR': '250',
    'WEATHER_OVERCAST_COLOR': '244',
    'WEATHER_SNOW_COLOR': '60',
    'WEATHER_THUNDER_COLOR': '226',
    'WEATHER_DEFAULT_COLOR': '195',
    'WEATHER_HIGH_TEMP_COLOR': '196',
    'WEATHER_MIDDLE_TEMP_COLOR': '209',
    'WEATHER_LOW_TEMP_COLOR': 'blue',
    'WEATHER_SHOW_UPDATE_TIME': '120',
    'WEATHER_UPDATE_TIME_COLOR': 'red',
}
DATA_PATH_FORMAT = '/tmp/thunderbolt100k_{0}.data'
PID_PATH = '/tmp/thunderbolt100k.pid'


def init():
    global CONFIG
    # Read configuration first
    CONFIG = load_conf()

    # Use default config if not specified
    for key in DEFAULT_CONFIG:
        if key not in CONFIG:
            CONFIG[key] = DEFAULT_CONFIG[key]


init()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import setuptools

from thunderbolt100k import __VERSION__

setuptools.setup(
    name='thunderbolt100k',
    version=__VERSION__,
    description='A cmdline tool to poll some info into local storage for later use',
    packages=setuptools.find_packages(),
    include_package_data=True,  # Include the template files
    test_suite='tests',
    author='Curtis Yu',
    author_email='icyarm@icloud.com',
    install_requires=[
        'schedule',
        'requests',
        'python-daemon',
    ],
    url='https://github.com/cuyu/thunderbolt100k',
    entry_points={
        "console_scripts": [
            "thunderbolt100k = thunderbolt100k.main:main",
        ],
    },
)



import requests
from thunderbolt100k import constants
import json
from thunderbolt100k.libs.compare_time import compare_time
from thunderbolt100k.libs.common import pl9k_color
import datetime

SESSION = requests.session()


def user_input():
    result = dict()
    result['WEATHER_TOKEN'] = raw_input("Please input your api key for https://api.apixu.com: ")
    result['WEATHER_CITY'] = raw_input('Please input the city of your location (e.g. Shanghai): ')
    return result


def fetch():
    assert 'WEATHER_TOKEN' in constants.CONFIG, 'You must set `THUNDERBOLT_WEATHER_TOKEN` in `~/.zshrc` file'
    assert 'WEATHER_CITY' in constants.CONFIG, 'You must set `THUNDERBOLT_WEATHER_CITY` in `~/.zshrc` file'
    response = SESSION.get(
        'https://api.apixu.com/v1/forecast.json?key={token}&q={city}&days=2'.format(
            token=constants.CONFIG.get('WEATHER_TOKEN'),
            city=constants.CONFIG.get('WEATHER_CITY')))

    return response.content if response.status_code == 200 else None


def display():
    path = constants.DATA_PATH_FORMAT.format('weather')
    with file(path, 'r') as f:
        content = json.load(f)
    now = datetime.datetime.now()
    # Use the next day's forecast info if the time is later than 23:00
    if now.hour >= 23:
        info = content['forecast']['forecastday'][1]['hour'][now.hour - 23]
    else:
        info = content['forecast']['forecastday'][0]['hour'][now.hour + 1]

    temp = info['temp_c']
    condition = info['condition']['text']
    update_time = content['current']['last_updated']
    delta_minute, delta_hour, delta_day = compare_time(update_time, '%Y-%m-%d %H:%M')

    if 'thunder' in condition or 'Thunder' in condition:
        symbol = "\uf0e7"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_THUNDER_COLOR'))
    elif 'rain' in condition or 'drizzle' in condition:
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_RAIN_COLOR'))
        symbol = "\ue239"
    elif 'Cloudy' in condition:
        symbol = "\uf0c2"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_CLOUD_COLOR'))
    elif 'Overcast' in condition:
        symbol = "\uf0c2"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_OVERCAST_COLOR'))
    elif "Partly cloudy" in condition:
        symbol = "\ue21d"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_CLOUD_COLOR'))
    elif condition == 'Sunny' or condition == 'Clear':
        symbol = "\uf185"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_SUN_COLOR'))
    elif 'snow' in condition:
        symbol = "\uf2dc"
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_SNOW_COLOR'))
    else:
        symbol_color = pl9k_color(constants.CONFIG.get('WEATHER_DEFAULT_COLOR'))
        symbol = "\uf422"

    if temp >= 35:
        temp_color = pl9k_color(constants.CONFIG.get('WEATHER_HIGH_TEMP_COLOR'))
        temp_symbol = '\uf2c7'
    elif temp >= 28:
        temp_color = pl9k_color(constants.CONFIG.get('WEATHER_MIDDLE_TEMP_COLOR'))
        temp_symbol = '\uf2c8'
    elif temp < 10:
        temp_color = pl9k_color(constants.CONFIG.get('WEATHER_LOW_TEMP_COLOR'))
        temp_symbol = '\uf2cb'
    else:
        temp_color = pl9k_color(constants.CONFIG.get('WEATHER_DEFAULT_COLOR'))
        temp_symbol = '\uf2c9'

    delay_minutes = delta_minute + 60 * delta_hour + 60 * 24 * delta_day

    if delta_day == 0:
        if delta_hour == 0:
            time_string = '>{0} min'.format(delta_minute)
        else:
            time_string = '>{0} hour'.format(delta_hour)
    else:
        time_string = '>{0} day'.format(delta_day)

    result = "%{" + symbol_color + "%}" + symbol + "  %{" + temp_color + "%}" + str(
        temp) + temp_symbol

    if delay_minutes >= int(constants.CONFIG.get('WEATHER_SHOW_UPDATE_TIME')):
        result += "%{" + pl9k_color(constants.CONFIG.get('WEATHER_UPDATE_TIME_COLOR')) + "%}" + "({0})".format(
            time_string)

    return result
